#include <avr/io.h>
#include <avr/interrupt.h>
#include <homelab/pin.h>
#include <homelab/delay.h>
#include <homelab/module/motors.h>
#include <homelab/module/lcd_alpha.h>
#include <stdio.h>

#define R0 10
#define R1 15

pin pin_trigger = PIN(E, 1);
pin pin_echo    = PIN(E, 0);
pin pin_trigger1 = PIN(E, 3);
pin pin_echo1  = PIN(E, 2);

static unsigned short clock=0;

#define ULTRASONIC_SPEED_OF_SOUND 34326UL // cm/s

unsigned short ultrasonic_measure(pin trigger, pin echo)
{
    // Viikude seadistus
    // Taimer 3 normaalreiimi
    // perioodiga F_CPU / 8

    timer3_init_normal(TIMER3_PRESCALE_8);

    // Pstiku viik krgeks
    pin_set(trigger);

    // Taimeri nullimine
    timer3_overflow_flag_clear();
    timer3_set_value(0);

    // ~10 us ootamine
    while (timer3_get_value() < 18) {}

    // Pstiku viik madalaks
    pin_clear(trigger);

    // Kaja signaali alguse ootamine
    while (!pin_get_value(echo))
    {
        // Liiga kaua oodatud ?
        if (timer3_overflow_flag_is_set())
        {
            return 0;
        }
    }

    // Taimeri nullimine
    timer3_set_value(0);

    // Kaja signaali lpu ootamine
    while (pin_get_value(echo))

    {
        // Liiga kaua oodatud ?
        if (timer3_overflow_flag_is_set())
        {
            return 0;
        }
    }

    // Mdetud aja kauguseks teisendamine
    //   kaugus = aeg * (1 / (F_CPU / 8)) * heli kiirus / 2
    unsigned long result=(unsigned long)timer3_get_value()*ULTRASONIC_SPEED_OF_SOUND / (F_CPU / 4);
    if(result>500)
        return 500;
    else
        return result;
}

unsigned short vertdist;
unsigned short horzdist;
unsigned short dist[2];
unsigned short filter(unsigned short distance, pin trigger, pin echo)

{
    dist[0]=ultrasonic_measure(trigger,echo);
    if(dist[0]==0)
        return distance;
    short d=100*(dist[0]-distance)/dist[0];//vahe protsentides
    if(d*d<400)//kui viksem kui 20%
        return (dist[0]+distance)/2;//aritm keskmine
    sw_delay_ms(100);
    dist[1]=ultrasonic_measure(trigger,echo);//kui suurem kui 20% mda veel ks
    d=100*(dist[1]-distance)/dist[1];
    if(d*d<400)//kui viksem kui 20%
        return (dist[1]+distance)/2;// stuff
    else
        return (dist[1]+dist[0])/2;
}

bool legsOpen=false;

ISR(TIMER0_COMP_vect)
{
   if(TCCR0&(1<<COM00))
        PORTD|=1;
    else
        PORTD&=~(1);
}

ISR(TIMER0_OVF_vect)
{
    if(TCCR0&(1<<COM00))
        PORTD&=~(1);
    else
       PORTD|= (1);
}

ISR(TIMER2_COMP_vect)
{
    cli();
    clock+=1;
    sei();
}

inline bool get_value(unsigned char port, unsigned char pin)
{
   return ((port)&(1<<pin))>0? true: false;
}

inline void ForWard()
{
   TIMSK |= ((1<<OCIE0) | (1<<TOIE0));
    PORTB&=~(1<<7);
    TCCR0|=(1<<COM01);
    TCCR0&=~(1<<COM00);
    PORTD&=~(2);
}

inline void BackWard()
{
    TIMSK |= ((1<<OCIE0) | (1<<TOIE0));
    PORTB|=(1<<7);
    TCCR0|=(1<<COM01);
    TCCR0|=(1<<COM00);
    PORTD|=(2);
}
inline void TurnRight()

{
    PORTB &= ~(1<<7);
    TCCR0|=(1<<COM01);
    TCCR0 &= ~(1<<COM00);
    PORTD &= ~(2);
    TIMSK &= ~((1<<OCIE0) | (1<<TOIE0));
}




inline void TurnLeft()
{
    TIMSK |= ((1<<OCIE0) | (1<<TOIE0));
    PORTB&=~(1<<7);
    TCCR0&= ~(1<<COM01);
    TCCR0 &= ~(1<<COM00);
    PORTD &= ~(2);
}

inline void Stop()
{
    TCCR0&= ~(1<<COM01);
    TCCR0 &= ~(1<<COM00);
    TIMSK &= ~((1<<OCIE0) | (1<<TOIE0));
    PORTD&=~(2);
    PORTD&=~(1);
    PORTB &= ~(1<<7);
}

bool OpenLegs()
{
    Stop();
        //OCR0=120;
    BackWard();
    sw_delay_ms(600);
    OCR1A=2900;
    sw_delay_ms(500);
    OCR1A=1843;
    //OCR0=255;
    Stop();
    sw_delay_ms(1000);
    legsOpen=true;
    return true;
}

bool CloseLegs()
{
    Stop();
    //OCR0=120;
    ForWard();
    sw_delay_ms(200);
    OCR1A=2900;
    sw_delay_ms(500);
    OCR1A=1843;
    //OCR0=255;
    Stop();
    sw_delay_ms(1000);
    legsOpen=false;
    return true;
}


int main()
{
    //taimeri seadistamine
	TCCR0|=(1<<COM01)|(1<<WGM00)|(1<<WGM01)|0x05;
    OCR0=255;
    
	//lcd ekraani initsialiseerimine
    lcd_alpha_init(LCD_ALPHA_DISP_ON);
    lcd_alpha_clear();
    char text[16];

    vertdist=ultrasonic_measure(pin_trigger, pin_echo);    
    horzdist=ultrasonic_measure(pin_trigger1, pin_echo1);

    unsigned short mindist=500;
    unsigned short maxdist=0;
    unsigned short openmindist=500;
    unsigned short openmaxdist=0;
    
	//Viikude seadistamine
    DDRC|=0x38;
    PORTC |=(0x38);
    DDRE|=(1<<PE4)|(1<<PE5);
    PORTE|=(1<<PE5);
    PORTE &= ~(1<<PE4);
    TCCR1A|=(1<<COM1A1)|(1<<COM1B1)|(1<<WGM11);
    TCCR1B|=(1<<WGM13)|(1<<WGM12)|(1<<CS11);
    ICR1=36864;
    OCR1A=1843;
    OCR1B=1843;

    pin_setup_output(pin_trigger);
    pin_setup_input_with_pullup(pin_echo);
    pin_setup_output(pin_trigger1);
    pin_setup_input_with_pullup(pin_echo1);

    TCCR2|=(1<<WGM21)|(1<<CS20)|(1<<CS22);
    OCR2=144;
    TIMSK|=(1<<OCIE2);

    dcmotor_init(0);
    dcmotor_init(1);
    servomotor_init(0);
    servomotor_init(1);
    sei();

    bool flag2=true;
    bool flag1=true;
    bool flag0=true;
    bool roaming=false;
    bool potncollision=false;
    bool collided=false;

    unsigned short t0=clock;
    unsigned short t1=clock;
    unsigned short t2;

    Stop();
    sw_delay_ms(2000);
    
    //philine programmitskkel
	while(1)
    {
        if(t0<clock)
        {           
        
            vertdist=filter(vertdist,pin_trigger, pin_echo);
            sw_delay_ms(100);
            horzdist=filter(horzdist,pin_trigger1, pin_echo1);
            if(!legsOpen)
            {
                if(vertdist>maxdist)
                    maxdist=vertdist;
                else if(vertdist<mindist)
                    mindist=vertdist;
            }
            else
            {
                if(vertdist>openmaxdist)
                    openmaxdist=vertdist;
                else if(vertdist<openmindist)
                    openmindist=vertdist;
            }
            t0=clock+10;
        }
        if(bit_is_clear(PINC,2)&&flag2)//punase led juures
        {
            flag2=false;
            OpenLegs();
        }
        else if(bit_is_set(PINC,2))
            flag2=true;
        if(bit_is_clear(PINC,1)&&flag1)//kollane
        {
            flag1=false;
            CloseLegs();
        }
        else if(bit_is_set(PINC,1))
            flag1=true;
        if(bit_is_clear(PINC,0)&&flag0)//roheline
        {
            flag0=false;
            static char count=1;
            
            mindist=500;
            maxdist=0;
            openmindist=500;
            openmaxdist=0;
            if(count&0x01)
            {
                roaming=true;//rita autonoomslet ttada
                count++;
            }
            else
            {
                roaming=false;
                count++;
            }
        }
         else if(bit_is_set(PINC,0))
            flag0=true;
        
        if(roaming)
        {
            if(horzdist<50)//kui 50 cm midagi paistab
            {                
                potncollision=true;
            }
            
            if(horzdist<20)//kui krge takistus (20 cm)
            {
                PORTC &= ~(0x10);
                Stop();
                sw_delay_ms(50);
                BackWard();
                sw_delay_ms(1250);
                OCR0=255;

                TurnLeft();
                sw_delay_ms(1500);
                Stop();

                potncollision=false;
                PORTC |= 0x10;
            }

            else if(!legsOpen&&vertdist<8)//90?   kui rattad kinni ja neb takistus
            {
                PORTC &= ~(0x20);
                OCR0=255;
                OpenLegs();                ;
                PORTC |= (0x20);
            }

            else if(!collided&&legsOpen&&vertdist<10)//kui prkub kokku takistusega peale ratatste avamist
                collided=true;//abi flag

            else if(collided&&legsOpen&&vertdist>12)//120? kui letanud takistuse
            {  
                PORTC &= ~(0x08);
                potncollision=false;

                OCR0=255;
                collided=false;
                CloseLegs();                
                PORTC |= (0x08);
            }
            else
            {                
                if(legsOpen)
                    OCR0=255;
                else
                    OCR0=100;
                ForWard();
            }            
        }

        else
        {
            Stop();
            OCR0=255;
        }
        
        sprintf(text,"l%dc%dp%dr%d %d",legsOpen,collided,potncollision,roaming,OCR0);
        lcd_alpha_goto_xy(0, 0);
        lcd_alpha_write_string(text);
        sprintf(text,"v%d h%d  ",vertdist,horzdist);
        lcd_alpha_goto_xy(0, 1);
        lcd_alpha_write_string(text);
     }
    return 0;
}
